/* ---------------------------------------------------------
  Compute the depth of a surface crevasse using the
  van der Veen (1998a) approach. Written by Stephen Jimenez.
------------------------------------------------------------ */


/* Include libraries. */
#include <iostream>
#include <math.h>


/* Define constants. */
const double rho_ice = 917; // ice density (kg/m^3)
const double rho_srf = 917; // surface density (kg/m^3)
const double rho_H2O = 1020; // seawater density  (kg/m^3)
const double C = 0.02; // density constant (m^-1)
const double g = 9.81; // gravity acceleration (m/s^2)
const double pi = M_PI; // pi
const double E = 9500; // Young's modulus for ice (GPa)
const double nu = 0.35; // Poisson's ratio for ice
const double mu = E/2/(1+nu); // shear modulus for ice (GPa)
const double KIc = 0.1; // critical stress intensity factor


/* Forward declarations. */
double Bisection(double H, double hw, double hs);
double dKI2(double b, double ds, double H);
double dKI3(double b, double ds, double H, double a);
double F(double lambda);
double G(double lambda, double gamma);
double KI1(double ds, double H, double Rxx);
double KI2(double ds, double H);
double KI3(double ds, double H, double hs);
double KInet(double ds, double H, double hs, double Rxx);
double sigma_l(double b);


/* ---------------------------------------------------------
  Define subroutines.
------------------------------------------------------------ */


// bisection method to compute crevasse depth
double Bisection(double H, double hw, double hs) {
  double Rxx = (0.5*rho_ice*g*H - 0.5*rho_H2O*g*hw*hw/H); // restrictive stress
  double da = 1e-6, db = 0.5*H, dc = H*(1-1e-6); // initial bounds for bisection
  double KI;
  double error = 1, tol = 1e-6;
  int k = 0, maxk = 50; // iterations
  while (fabs(error) > tol && k < maxk) {
    KI = KInet(db, H, hs, Rxx);
    if (KI > 0) da = db;
    else dc = db;
    db = (da + dc)/2;
    error = fabs(da - dc)/H;
    k++; // update iteration count
  }
  return db;
}


// KI2 integrand (Equation 11)
double dKI2(double b, double ds, double H) {
  double lambda = ds/H,  gamma = b/ds;
  return 2*sigma_l(b)/sqrt(pi*ds)*G(lambda, gamma);
}


// KI3 integrand (integrand of Equation 16)
double dKI3(double b, double ds, double H, double a) {
  double lambda = ds/H,  gamma = b/ds;
  return (b - a)*G(lambda, gamma);
}


// geometry factor for KI1 (Equation 6)
double F(double lambda) {
  return (1.12 - 0.23*lambda + 10.55*pow(lambda, 2) - 21.72*pow(lambda, 3)
    + 30.39*pow(lambda, 4));
}


// geometry factor for KI2 and KI3 (Equation 12)
double G(double lambda, double gamma) {
  double term1 = 3.52*(1 - gamma)/pow((1 - lambda), 1.5);
  double term2 = (4.35 - 5.28*gamma)/sqrt(1 - lambda);
  double term3 = (1.3 - 0.3*pow(gamma, 1.5))/sqrt(1 - pow(gamma, 2)) + 0.83 - 1.76*gamma;
  double term4 = (1 - (1 - gamma)*lambda);
  return term1 - term2 + term3*term4;
}


// stress intensity factor from Rxx (Equation 5)
double KI1(double ds, double H, double Rxx) {
  double lambda = ds/H;
  return F(lambda)*Rxx*sqrt(pi*ds);
}


// stress intensity factor from lithostatic pressure (Equation 13)
double KI2(double ds, double H) {
  int n = 1e3;
  double b = 0, db = ds/n, Ksum = 0;
  double y1, y0;
  y0 = dKI2(b, ds, H);
  // integrate using trapezoidal rule
  for (int i=1; i<n; i++) {
    b += db;
    y1 = dKI2(b, ds, H);
    Ksum += 0.5*(y1 + y0)*db;
    y0 = y1;
  }
  return Ksum;
}


// stress intensity factor from water pressure in crevasse (Equation 16)
double KI3(double ds, double H, double hs) {
  double a = (1 - hs)*ds;
  if (ds <= a) return 0;
  else {
    int n = 1e3;
    double b = a, db = (ds-a)/n, Ksum = 0;
    double y1, y0;
    y0 = dKI3(b, ds, H, a);
    // integrate using trapezoidal rule
    for (int i=1; i<n; i++) {
      b += db;
      y1 = dKI3(b, ds, H, a);
      Ksum += 0.5*(y1 + y0)*db;
      y0 = y1;
    }
    return 2*rho_H2O*g/sqrt(pi*ds)*Ksum;
  }
}


// net stress intensity factor
double KInet(double ds, double H, double hs, double Rxx) {
  return KI1(ds, H, Rxx) + KI2(ds, H) + KI3(ds, H, hs) - KIc*1e6;
}


// overburden stress (Equation 9)
double sigma_l(double b) {
  return -rho_ice*g*b + (rho_ice - rho_srf)/C*g*(1 - exp(-C*b));
}


// main function
int main() {
  // user input
  double H, hw, hs, ds;
  std::cout << "\n  Height of ice slab (meters) = ";
  std::cin >> H;
  std::cout << "  Water level at terminus (percent) = ";
  std::cin >> hw;
  std::cout << "  Water level in crevasse (percent) = ";
  std::cin >> hs;
  // compute crevasse depth
  hw *= (0.01*H);
  hs *= 0.01;
  ds = Bisection(H, hw, hs);
  std::cout << "\n  Absolute crevasse depth:   " << round(ds*1e2)/1e2 << " m" << std::endl;
  std::cout << "  Normalized crevasse depth: " << round(ds/H*1e4)/1e2 << " %";
  std::cout << std::endl << std::endl;

  return 0;
}
